/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 *  Authors: Iain Holmes <iain@prettypeople.org>
 *
 *  Copyright 2003 Iain Holmes
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of version 2 of the GNU General Public 
 *  License as published by the Free Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *
 */

#include <marlin/marlin-program.h>
#include <marlin/marlin-marshal.h>

enum {
	NEW_WINDOW,
	CLIPBOARD_CHANGED,
	LAST_SIGNAL
};

static GObjectClass *parent_class = NULL;
static MarlinProgram *main_program = NULL;
static guint signals[LAST_SIGNAL];

struct _MarlinProgramPrivate {
	MarlinSample *clipboard;
};

static void
finalize (GObject *object)
{
	MarlinProgram *program;
	MarlinProgramPrivate *priv;

	program = MARLIN_PROGRAM (object);
	priv = program->priv;
	
	g_free (priv);
	program->priv = NULL;

	parent_class->finalize (object);
}

static void
dispose (GObject *object)
{
	MarlinProgram *program;
	MarlinProgramPrivate *priv;
	
	program = MARLIN_PROGRAM (object);
	priv = program->priv;

	if (priv->clipboard != NULL) {
 		g_object_unref (G_OBJECT (priv->clipboard));
		priv->clipboard = NULL;
	}

	parent_class->dispose (object);
}

static void
class_init (MarlinProgramClass *klass)
{
	GObjectClass *object_class = (GObjectClass *) klass;

	object_class->finalize = finalize;
	object_class->dispose = dispose;

	parent_class = g_type_class_peek_parent (klass);

	signals[NEW_WINDOW] = g_signal_new ("new-window",
					    G_TYPE_FROM_CLASS (klass),
					    G_SIGNAL_RUN_FIRST |
					    G_SIGNAL_NO_RECURSE,
					    G_STRUCT_OFFSET (MarlinProgramClass, new_window),
					    NULL, NULL,
					    marlin_marshal_VOID__OBJECT,
					    G_TYPE_NONE, 1,
					    G_TYPE_OBJECT);
	signals[CLIPBOARD_CHANGED] = g_signal_new ("clipboard-changed",
						   G_TYPE_FROM_CLASS (klass),
						   G_SIGNAL_RUN_FIRST |
						   G_SIGNAL_NO_RECURSE,
						   G_STRUCT_OFFSET (MarlinProgramClass, clipboard_changed),
						   NULL, NULL,
						   g_cclosure_marshal_VOID__VOID,
						   G_TYPE_NONE, 0);
}

static void
init (MarlinProgram *program)
{
	program->priv = g_new0 (MarlinProgramPrivate, 1);
}

GType
marlin_program_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		GTypeInfo info = {
			sizeof (MarlinProgramClass), NULL, NULL,
			(GClassInitFunc) class_init, NULL, NULL,
			sizeof (MarlinProgram), 0, (GInstanceInitFunc) init
		};

		type = g_type_register_static (G_TYPE_OBJECT,
					       "MarlinProgram", &info, 0);
	}

	return type;
}

MarlinProgram *
marlin_program_get_default (void)
{
	if (main_program != NULL) {
		return main_program;
	}

	main_program = g_object_new (MARLIN_PROGRAM_TYPE, NULL);
	return main_program;
}

void
marlin_program_destroy_default (void)
{
	if (main_program == NULL) {
		return;
	}

	/* Should this be held in a mutex? */
	g_object_unref (main_program);
	main_program = NULL;
}

GObject *
marlin_program_get_clipboard (MarlinProgram *program)
{
	return program->priv->clipboard;
}

void
marlin_program_set_clipboard (MarlinProgram *program,
			      GObject *clipboard_object)
{
	if (program->priv->clipboard != NULL) {
		g_object_unref (program->priv->clipboard);
	}

	program->priv->clipboard = clipboard_object;
	if (program->priv->clipboard != NULL) {
		g_object_ref (program->priv->clipboard);
	}

	g_signal_emit (G_OBJECT (program), signals[CLIPBOARD_CHANGED], 0);
}
