/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 *  Authors: Iain Holmes <iain@prettypeople.org>
 *
 *  Copyright 2003 Iain Holmes
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of version 2 of the GNU General Public 
 *  License as published by the Free Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *
 */

#include <gtk/gtkdialog.h>
#include <gtk/gtkstock.h>

#include <libgnome/gnome-i18n.h>

#include <gst/gst.h>

#include <marlin/marlin-save-pipeline.h>
#include <marlin/marlin-a11y-helper.h>
#include <marlin/marlin-utils.h>
#include <marlin/marlin-x-utils.h>

#include "marlin-mp3-dialog.h"

typedef void (*MarlinCallback) (gpointer data);
struct _vorbis_data {
	MarlinPipeline *pipeline;
	GstElement *encoder;
};

static void
dialog_response (GtkDialog *dialog,
		 guint response_id,
		 struct _vorbis_data *vd)
{
	switch (response_id) {
	case GTK_RESPONSE_HELP:
		marlin_display_help ("marlin-vorbis-dialog");
		return;

	case GTK_RESPONSE_OK:
		marlin_pipeline_set_state (vd->pipeline, GST_STATE_PLAYING);
		break;

	default:
		marlin_pipeline_set_state (vd->pipeline, GST_STATE_NULL);
		break;
	}

	gtk_widget_destroy (GTK_WIDGET (dialog));
	g_free (vd);
}

void
marlin_vorbis_dialog_new (GtkWindow *parent,
			  MarlinPipeline *save_pipeline)
{
	MarlinSample *sample;
	struct _vorbis_data *vd;
	GtkWidget *dialog, *vbox;
	char *name, *title;
	
	vd = g_new (struct _vorbis_data, 1);
	vd->encoder = marlin_pipeline_get_element_by_name (save_pipeline, "encoder");
	g_assert (vd->encoder != NULL);

	vd->pipeline = save_pipeline;

	dialog = gtk_dialog_new ();
	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent));
	g_object_get (G_OBJECT (parent),
		      "sample", &sample,
		      NULL);

	g_object_get (G_OBJECT (sample),
		      "name", &name,
		      NULL);

	title = g_strdup_printf (_("Ogg Vorbis Encoding Parameters: %s"), name);
	g_free (name);

	gtk_window_set_title (GTK_WINDOW (dialog), title);
	g_free (title);

	vbox = gtk_vbox_new (FALSE, 12);
	gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), vbox, TRUE, TRUE, 0);
	gtk_widget_show (vbox);
	
	gtk_dialog_add_button (GTK_DIALOG (dialog),
			       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
	gtk_dialog_add_button (GTK_DIALOG (dialog),
			       GTK_STOCK_SAVE, GTK_RESPONSE_OK);
	gtk_dialog_add_button (GTK_DIALOG (dialog),
			       GTK_STOCK_HELP, GTK_RESPONSE_HELP);
	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
	g_signal_connect (dialog, "response",
			  G_CALLBACK (dialog_response), vd);

	gtk_widget_show (dialog);
}
